package jp.co.riiiver.sdk.sample

import android.app.AlertDialog
import android.content.DialogInterface
import android.content.Intent
import android.graphics.Bitmap
import android.graphics.Color
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.CompoundButton
import androidx.fragment.app.Fragment
import com.riiiver.sdk.ersdk.ERApplet
import com.riiiver.sdk.ersdk.ERError
import jp.co.riiiver.sdk.sample.piececore.TriggerPieceCoreSample
import jp.co.riiiver.sdk.sample.piececore.TriggerPieceCoreSample2
import kotlinx.android.synthetic.main.fragment_iiidea_info.*
import timber.log.Timber
import java.util.*


class IiideaInfoFragment : Fragment() {

    var iiidea: ERApplet? = null

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        return inflater.inflate(R.layout.fragment_iiidea_info, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        Timber.d("in")

        // タイトルをセット
        title.text = iiidea!!.applet.title.get(Locale.getDefault())

        activate.isChecked = iiidea!!.isActivated

        setTriggerButtonEnabled()

        // アイコンをセット
        RiiiverSDKManager.getImage(iiidea!!.applet.iconUrl) { bitmap: Bitmap?, _: String?, erError: ERError? ->
            if (erError != null) {
                Timber.e("error %s", erError.message)
                return@getImage
            }
            icon.setImageBitmap(bitmap)
        }

        activate.setOnCheckedChangeListener { _: CompoundButton, isChecked: Boolean ->
            if (isChecked) {
                RiiiverSDKManager.activate(requireContext(), iiidea!!) { isSuccess: Boolean, erError: ERError? ->
                    if (!isSuccess) {
                        Timber.e("activate failed")
                        if (erError != null && erError.erErrorCode == ERError.ERErrorCode.ER_ERROR_CODE_PERSONAL_INFO_ERROR) {
                            activate.isChecked = false
                            goToPersonalInfoFragment()
                        }
                        else {
                            activateFailedAlert(erError)
                        }
                        return@activate
                    }
                    setTriggerButtonEnabled()
                }
            } else {
                RiiiverSDKManager.deactivate(iiidea!!) { isSuccess: Boolean, erError: Exception? ->
                    if (!isSuccess) {
                        Timber.e("deactivate failed")
                        activateFailedAlert(erError)
                        return@deactivate
                    }
                    Timber.d("deactivate succeeded.")
                    setTriggerButtonEnabled()
                }
            }
        }

        pieceCore1.setOnClickListener {
            Timber.d("pieceCore1")
            val text = output_edit!!.text.toString()
            val intent = Intent(TriggerPieceCoreSample.ACTION)
            intent.putExtra("identifier", iiidea!!.id)
            intent.putExtra("output", text)
            requireActivity().sendBroadcast(intent)
        }

        pieceCore2.setOnClickListener {
            Timber.d("pieceCore2")
            val text = output_edit!!.text.toString()
            val intent = Intent(TriggerPieceCoreSample2.ACTION)
            intent.putExtra("identifier", iiidea!!.id)
            intent.putExtra("output", text)
            requireActivity().sendBroadcast(intent)
        }

        back.setOnClickListener {
            Timber.d("logout")
            back(false)
        }

        preference.setOnClickListener {
            Timber.d("preference")
            goToSettingFragment()
        }

        val appletIdList = RiiiverSDKConfig.getAppletIdList()
        appletIdList.forEach {
            if (iiidea!!.applet.id == it) {
                // 強制ダウンロードしたiiideaは削除できないようにする
                remove.isEnabled = false
            }
        }

        remove.setOnClickListener {
            Timber.d("remove")
            removeAlert()
        }

    }

    private fun removeAlert() {
        AlertDialog.Builder(context)
            .setTitle("Delete the iiidea?")
            .setMessage("You can download the iiidea you deleted from the store again.")
            .setPositiveButton("OK") { _: DialogInterface, _: Int ->
                RiiiverSDKManager.removeMyIiidea(iiidea!!) { isSuccess: Boolean, error: ERError? ->
                    if (!isSuccess) {
                        Timber.e("remove failed")
                        removeFailedAlert(error)
                    } else {
                        back(true)
                    }
                }
            }
            .setNegativeButton("Cancel", null)
            .show()
    }

    private fun removeFailedAlert(error: ERError?) {
        var message = ""
        if (error != null) {
            message = error.message.toString()
        }
        AlertDialog.Builder(context)
            .setTitle("Delete Failed")
            .setMessage(message)
            .setPositiveButton("OK", null)
            .show()

    }

    private fun activateFailedAlert(error: Exception?) {
        var message = ""
        if (error != null) {
            message = error.message.toString()
        }
        activateFailedAlert(message)
    }

    private fun activateFailedAlert(error: ERError?) {
        var message = ""
        if (error != null) {
            message = error.message.toString()
        }
        activateFailedAlert(message)
    }

    private fun activateFailedAlert(message: String) {
        AlertDialog.Builder(context)
            .setTitle("Activate Failed")
            .setMessage(message)
            .setPositiveButton("OK") { _: DialogInterface, _: Int -> activate.isChecked = false }
            .show()
    }

    private fun setTriggerButtonEnabled() {
        val isActivated = iiidea!!.isActivated
        if (iiidea!!.triggerBlockMetadata.executor == "TriggerPieceCoreSample") {
            pieceCore1.isEnabled = isActivated
            pieceCore2.isEnabled = false
        } else if (iiidea!!.triggerBlockMetadata.executor == "TriggerPieceCoreSample2") {
            pieceCore1.isEnabled = false
            pieceCore2.isEnabled = isActivated
        }
    }

    private fun goToSettingFragment() {
        val nextFragment = IiideaSettingFragment()
        nextFragment.iiidea = iiidea

        val transaction = requireActivity().supportFragmentManager.beginTransaction()
        transaction.hide(this)
        transaction.add(R.id.container, nextFragment)
        transaction.addToBackStack(null)
        transaction.commit()
    }

    private fun goToPersonalInfoFragment() {
        val nextFragment = PersonalInfoFragment()
        nextFragment.iiidea = iiidea

        val transaction = requireActivity().supportFragmentManager.beginTransaction()
        transaction.hide(this)
        transaction.add(R.id.container, nextFragment)
        transaction.addToBackStack(null)
        transaction.commit()
    }

    private fun back(isReload: Boolean) {
        val transaction = requireActivity().supportFragmentManager.beginTransaction()
        if (!isReload) {
            parentFragmentManager.popBackStack()
        } else {
            transaction.replace(R.id.container, IiideaListFragment())
            transaction.commit()
        }
    }
}