package jp.co.riiiver.sdk.sample

import android.content.Context
import android.graphics.Bitmap
import android.graphics.Color
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import android.widget.AdapterView.OnItemClickListener
import androidx.fragment.app.Fragment
import com.riiiver.sdk.ersdk.ERApplet
import com.riiiver.sdk.ersdk.ERError
import kotlinx.android.synthetic.main.fragment_iiidea_list.*
import timber.log.Timber
import java.util.*


class IiideaListFragment : Fragment() {
    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        return inflater.inflate(R.layout.fragment_iiidea_list, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        if (!RiiiverSDKManager.authenticated) {
            // ログアウトされている場合
            requireActivity().supportFragmentManager.beginTransaction().remove(this).commit()
            return
        }

        profile.setOnClickListener {
            Timber.d("profile")
            goToProfileFragment()
        }
        news.setOnClickListener {
            Timber.d("news")
            goToNewsFragment()
        }
        store.setOnClickListener {
            Timber.d("store")
            goToStoreFragment()
        }

        progress_bar.visibility = ProgressBar.VISIBLE
        RiiiverSDKManager.getMyApplets { iiideaList: MutableList<ERApplet>?, _: Exception? ->
            if (iiideaList == null || progress_bar == null) {
                return@getMyApplets
            }
            progress_bar.visibility = ProgressBar.INVISIBLE
            val iiideas = iiideaList.toList()
            // ListViewをセット
            val adapter = AppletArrayAdapter(requireContext(), R.layout.fragment_iiidea_list, iiideas)
            val listView: ListView = view.findViewById<View>(R.id.list_view) as ListView
            listView.adapter = adapter
            // セルを選択されたら詳細画面フラグメント呼び出す
            listView.onItemClickListener = OnItemClickListener { _, _, position, _ ->
                goToInfoFragment(iiideas[position])
            }
        }
    }

    override fun onResume() {
        super.onResume()
        RiiiverSDKManager.isUnreadNews{ isUnread: Boolean, _: ERError? ->
            if(isUnread) {
                unread.visibility = View.VISIBLE
            }
            else {
                unread.visibility = View.INVISIBLE
            }
        }
    }

    class AppletArrayAdapter(context: Context, resID: Int, private val items: List<ERApplet>) :
        ArrayAdapter<ERApplet?>(context, resID, items) {
        private val inflater: LayoutInflater =
            context.getSystemService(Context.LAYOUT_INFLATER_SERVICE) as LayoutInflater

        override fun getView(
            position: Int,
            convertView: View?,
            parent: ViewGroup
        ): View {
            val root = null
            val view: View =
                convertView ?: inflater.inflate(R.layout.listview_iiidea_list_cell, root)

            val iiidea: ERApplet = items[position]

            // タイトルをセット
            val title = view.findViewById<View>(R.id.cell_title) as TextView
            title.text = iiidea.applet.title.get(Locale.getDefault())
            title.setTextColor(Color.WHITE)
            val appletIdList = RiiiverSDKConfig.getAppletIdList()
            appletIdList.forEach(){
                if (iiidea.applet.id == it) {
                    title.setTextColor(Color.RED)
                }
            }

            // アイコンをセット
            val icon: ImageView = view.findViewById<View>(R.id.cell_icon) as ImageView
            RiiiverSDKManager.getImage(iiidea.applet.iconUrl) { bitmap: Bitmap?, _: String?, erError: ERError? ->
                if (erError != null) {
                    Timber.e("error %s", erError.message)
                    return@getImage
                }
                icon.setImageBitmap(bitmap)
            }
            return view
        }

    }

    private fun goToInfoFragment(iiidea: ERApplet) {
        val nextFragment = IiideaInfoFragment()
        nextFragment.iiidea = iiidea

        val transaction = requireActivity().supportFragmentManager.beginTransaction()
        transaction.hide(this)
        transaction.add(R.id.container, nextFragment)
        transaction.addToBackStack(null)
        transaction.commit()
    }

    private fun goToStoreFragment() {
        val transaction = requireActivity().supportFragmentManager.beginTransaction()
        transaction.replace(R.id.container, StoreFragment())
        transaction.addToBackStack(null)
        transaction.commit()
    }

    private fun goToNewsFragment() {
        val transaction = requireActivity().supportFragmentManager.beginTransaction()
        transaction.replace(R.id.container, NewsInfoFragment())
        transaction.addToBackStack(null)
        transaction.commit()
    }

    private fun goToProfileFragment() {
        val transaction = requireActivity().supportFragmentManager.beginTransaction()
        transaction.replace(R.id.container, ProfileFragment())
        transaction.addToBackStack(null)
        transaction.commit()
    }
}