package jp.co.riiiver.sdk.sample

import android.Manifest.permission.*
import android.content.pm.PackageManager.PERMISSION_GRANTED
import android.content.Context
import android.content.DialogInterface
import android.content.Intent
import android.content.pm.PackageManager
import android.graphics.Color
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.os.Handler
import android.provider.Settings
import android.view.*
import android.view.ViewGroup.LayoutParams.MATCH_PARENT
import android.widget.LinearLayout
import androidx.appcompat.app.AlertDialog
import androidx.browser.customtabs.CustomTabsIntent
import androidx.core.app.NotificationManagerCompat
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import androidx.preference.PreferenceManager
import com.riiiver.sdk.ersdk.ERApplet
import com.riiiver.sdk.ersdk.ERError
import com.riiiver.sdk.ersdk.preferences.ERPreferencesView
import com.riiiver.sdk.ersdk.preferences.ERPreferencesViewListener
import com.riiiver.sdk.ersdk.preferences.ERPreferencesViewPermissionListener
import jp.co.riiiver.sdk.sample.RiiiverSDKConfig.riiiverAuthUri
import kotlinx.android.synthetic.main.fragment_iiidea_setting.*
import kotlinx.android.synthetic.main.fragment_store.back
import timber.log.Timber
import java.io.File
import java.io.FileOutputStream
import java.net.URL

class IiideaSettingFragment : Fragment(), ERPreferencesViewListener,
    ERPreferencesViewPermissionListener {

    var PERMISSION_CODE = 1000

    private var isOnCreateRun: Boolean = false

    private lateinit var preferencesView: ERPreferencesView
    var iiidea: ERApplet? = null

    companion object {

        private var root = "preferences"

        fun initialize(context: Context) {
            Timber.d("initialize in: context=$context")
            // AssetからPreferences用WebコンテンツのPathを取得
            val paths = getAssetFilePaths(context, root)
            // アプリ配下にPreferences用Webコンテンツをコピー
            copyAssetFiles(context, paths)
        }

        /**
         * 指定したアセットディレクトリの配下に属するファイルパスを全て取得する
         * @param context Context
         * @param assetDirectory アセット内のディレクトリ名
         * @return ファイルパスリスト
         */
        private fun getAssetFilePaths(context: Context, assetDirectory: String): ArrayList<String> {
            Timber.d("getAssetFilePaths in: context=$context assetDirectory=$assetDirectory")
            val ret: ArrayList<String> = arrayListOf()
            val pathList: Array<String>? = context.assets.list(assetDirectory)
            pathList?.forEach { path ->
                // 文字列検索
                if (Regex("\\.").containsMatchIn(path)) {
                    // "."が含まれている場合はファイルと判断する
                    ret.add(assetDirectory + File.separator + path)
                } else {
                    // "."が含まれていない場合はディレクトリと判断して配下のファイルを取得する
                    ret.addAll(getAssetFilePaths(context, assetDirectory + File.separator + path))
                }
            }
            return ret
        }

        /**
         * 指定したアセットディレクトリ配下のファイルをアプリ配下の領域にコピーする
         * @param context Context
         * @param filePaths コピーするファイル名のリスト(パス付き)
         */
        private fun copyAssetFiles(context: Context, filePaths: ArrayList<String>) {
            Timber.d("copyAssetFiles in: context=$context filePaths=$filePaths")
            filePaths.forEach { path ->
                val appPath = getRootPath(context) + path
                val file = File(appPath)
                val filePath = File(file.parent!!)
                if (!filePath.mkdirs()) {
                    Timber.w("mkdirs failure. already $filePath")
                }
                context.assets.open(path).use { inputStream ->
                    FileOutputStream(file, false).use { outputStream ->
                        inputStream.copyTo(outputStream)
                    }
                }
            }
        }

        /**
         * アプリ保存領域のRootディレクトリのパスを取得する
         * @param context Context
         * @return Rootディレクトリのパス
         */
        private fun getRootPath(context: Context): String {
            Timber.d("getRootPath in: context=$context")
            return context.filesDir.parent!! + File.separator
        }
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        return inflater.inflate(R.layout.fragment_iiidea_setting, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        Timber.d("in")
        back.setOnClickListener {
            Timber.d("back")
            back()
        }

        if (iiidea != null) {
            // PreferencesのViewを生成
            preferencesView = ERPreferencesView(context)

            // Preference画面読み込み
            preferencesView.loadPreferences(iiidea, URL("file://" + getRootPath(requireContext()) + root))

            preferencesView.preferencesListener = this
            preferencesView.permissionListener = this

            preferencesView.layoutParams = LinearLayout.LayoutParams(MATCH_PARENT,MATCH_PARENT)

            //背景画像設定
            preferencesView.setBackgroundColor(Color.BLACK)
            webPreference.addView(preferencesView)
        } else {
            Timber.e("applet is null.")
        }
    }

    override fun onResume() {
        super.onResume()
        Timber.d("onResume in")
        if (isOnCreateRun) {
            Timber.d("Ignore onResume which is called immediately.")
            // onCreate後、すぐに呼ばれるonResumeは無視する
            isOnCreateRun = false
            return
        }
        // フォアグラウンドになったらPreferencesを更新
        reloadPreferences()
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<out String>, grantResults: IntArray) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        Timber.d("onRequestPermissionsResult in: requestCode=$requestCode permissions=$permissions grantResults=$grantResults")
        RiiiverSDKManager.showPermissionDialog(this.requireContext(), requestCode, permissions, grantResults)
        if (requestCode == PERMISSION_CODE) {
            if (grantResults.isNotEmpty() && grantResults[0] == PERMISSION_GRANTED) {
                // パーミッションに変化がある場合はPreferencesViewをリロード
                reloadPreferences()
            }
        }
    }

    override fun updateTriggerPreferences(p0: ERApplet?) {
        Timber.d("updateTriggerPreferences in")
    }

    override fun updateServicePreferences(p0: ERApplet?) {
        Timber.d("updateServicePreferences in")
    }

    override fun updateActionPreferences(p0: ERApplet?) {
        Timber.d("updateActionPreferences in")
    }

    override fun closePreferences(p0: ERApplet?) {
        Timber.d("closePreferences in")
        // PreferenceのWebContentに閉じるボタンを表示している場合に呼ばれる。
        // デフォルトでは呼ばれることはない。
    }

    override fun requestPermissionScreen(permissionType: String?) {
        Timber.d("requestPermissionScreen in: permissionType =$permissionType")
        val  permissions = ArrayList<String>()
        when (permissionType) {
            "location" -> {
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
                    permissions.add(ACCESS_BACKGROUND_LOCATION)
                }
                permissions.add(ACCESS_FINE_LOCATION)
                if(!isLocationPermission(permissions)){
                    requestPermission(permissions)
                }
            }
            "microphone" -> {
                permissions.add(RECORD_AUDIO)
                requestPermission(permissions)
            }
            "calendar" -> {
                permissions.add(READ_CALENDAR)
                requestPermission(permissions)
            }
            "notification" -> {
                requestNotificationPermission()
            }
            "notificationAccess" -> {
                requestNotificationAccessPermission()
            }
            else -> {
                Timber.e("permissionType Error: permissionType=$permissionType")
            }
        }

    }

    override fun requestAuthScreen(blockId: String?) {
        Timber.d("requestAuthScreen in: blockId=$blockId")
        if (blockId != null) {
            val dialog = AlertDialog.Builder(requireContext()).setMessage("Agree cooperation with service?")
            dialog.setPositiveButton("OK") { _: DialogInterface, _: Int ->
                RiiiverSDKManager.showAuthenticationSession(requireContext(), blockId) { isSuccess: Boolean, error: Exception? ->
                    if (error != null) {
                        Timber.e("error %s", error.message)
                        return@showAuthenticationSession
                    }
                    if(!isSuccess) {
                        val errorDialog = AlertDialog.Builder(requireContext()).setMessage("Display failed.")
                        errorDialog.setPositiveButton("OK", null)
                        errorDialog.show()
                    }
                }
            }
            dialog.setNegativeButton("CANCEL", null)
            dialog.show()
        }
    }

    override fun requestDeleteAuthScreen(blockId: String?) {
        Timber.d("requestDeleteAuthScreen in: blockId=$blockId")
        if (blockId != null) {
            val dialog = AlertDialog.Builder(requireContext()).setMessage("Stop cooperation with service?")
            dialog.setPositiveButton("OK") { _: DialogInterface, _: Int ->

                RiiiverSDKManager.stopAuthenticationSession(blockId) { isSuccess: Boolean, error: ERError? ->
                    if (error != null) {
                        Timber.e("error %s", error.message)
                        return@stopAuthenticationSession
                    }
                    if(!isSuccess) {
                        val errorDialog = AlertDialog.Builder(requireContext()).setMessage("Delete failed.")
                        errorDialog.setPositiveButton("OK", null)
                        errorDialog.show()
                    }
                    else {
                        reloadPreferences()
                    }
                }
            }
            dialog.setNegativeButton("CANCEL", null)
            dialog.show()
        }
    }

    // パーミッション許可
    private fun requestPermission(permission: ArrayList<String>) {
        Timber.d("requestPermission in: permission=$permission")
        val sharedPreferences = PreferenceManager.getDefaultSharedPreferences(context)
        val isFirst = sharedPreferences.getBoolean("isFirstRequestPermission", false)

        if (this.shouldShowRequestPermissionRationale(permission[0]) || !isFirst) {
            this.requestPermissions(permission.toTypedArray(), PERMISSION_CODE)
            val editor = sharedPreferences.edit()
            editor.putBoolean("isFirstRequestPermission", true)
            editor.apply()
        } else {
            // 「今後は確認しない」にチェックが入っている場合
            Handler().post {
                // システムのアプリ設定画面
                val intent = Intent(
                    Settings.ACTION_APPLICATION_DETAILS_SETTINGS,
                    Uri.parse("package:" + this.requireActivity().packageName)
                )
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                this.requireActivity().startActivity(intent)
            }
        }
    }

    // 位置情報パーミッションの確認
    private fun isLocationPermission(permission: ArrayList<String>): Boolean {
        Timber.d("isLocationPermission in: permission=$permission")
        permission.forEach {
            if (ContextCompat.checkSelfPermission(requireContext(), it) != PERMISSION_GRANTED) {
                return false
            }
        }
        // アプリの位置情報権限が既に設定されている状態でSDKから位置情報権限要求が発生した場合
        Handler().post {
            // システムの位置情報設定画面
            val intent = Intent(
                Settings.ACTION_LOCATION_SOURCE_SETTINGS
            )
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            this.requireActivity().startActivity(intent)
        }
        return true
    }

    // 通知パーミッション許可
    private fun requestNotificationPermission() {
        val result: Boolean = NotificationManagerCompat.from(requireContext()).areNotificationsEnabled()
        if(!result) {
            val dialog = AlertDialog.Builder(requireContext()).setMessage("Agree access to notification?")
            dialog.setPositiveButton("OK") { _: DialogInterface, _: Int ->
                Handler().post {
                    // システムのアプリ設定画面
                    val intent = Intent(
                        Settings.ACTION_APPLICATION_DETAILS_SETTINGS,
                        Uri.parse("package:" + this.requireActivity().packageName)
                    )
                    intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                    this.requireActivity().startActivity(intent)
                }
            }
            dialog.setNegativeButton("CANCEL", null)
            dialog.show()
        }
    }

    // 通知アクセスパーミッション許可
    private fun requestNotificationAccessPermission() {
        val dialog =
            AlertDialog.Builder(requireContext()).setMessage("Agree access to notification access?")
        dialog.setPositiveButton("OK") { _: DialogInterface, _: Int ->
            Handler().post {
                // 通知アクセス設定画面
                val intent = Intent(
                    Settings.ACTION_NOTIFICATION_LISTENER_SETTINGS
                )
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                this.requireActivity().startActivity(intent)
            }
        }
        dialog.setNegativeButton("CANCEL", null)
        dialog.show()
    }
    // Preferences画面更新
    private fun reloadPreferences() {
        Timber.d("reloadPreferences in")
        RiiiverSDKManager.getAWSToken { _: Boolean, _: Exception? ->
            preferencesView.reloadPreferences()
        }
    }

    private fun back() {
        parentFragmentManager.popBackStack()
    }
}