package jp.co.riiiver.sdk.sample

import android.content.Intent
import android.net.Uri
import android.os.Bundle
import androidx.appcompat.app.AppCompatActivity
import androidx.fragment.app.Fragment
import com.riiiver.auth.ERAuthToken
import com.riiiver.auth.IERAuthChallengeCallback
import com.riiiver.auth.IERAuthWebLoginCallback
import com.riiiver.sdk.ersdk.ERError
import timber.log.Timber
import timber.log.Timber.DebugTree


class MainActivity : AppCompatActivity() {

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        Timber.plant(DebugTree())

        setContentView(R.layout.activity_main)

        RiiiverSDKManager.initialize(applicationContext) { isLoggedIn: Boolean ->
            if (Intent.ACTION_VIEW == intent.action) {
                // ブラウザからWebトークン情報をSDKに登録する
                RiiiverSDKManager.challengeAuthByWebLogin(intent!!, object :
                    IERAuthChallengeCallback {
                    override fun onSuccess(authToken: ERAuthToken?) {
                        Timber.d("onSuccess in")
                        // 強制ダウンロード対象のiiideaを設定
                        RiiiverSDKManager.forcedDownloadApplet{ _: Boolean, _: ERError? ->
                            // Go to next View - 次の画面へ
                            transitionFragment(IiideaListFragment())
                        }
                    }

                    override fun onError(error: Exception) {
                        Timber.e("onError in: error %s", error.message)
                    }
                })
            } else {
                if (!isLoggedIn) {
                    // 未認証の場合はログイン画面を起動
                    startWebLogin()
                } else {
                    // 強制ダウンロード対象のiiideaを設定
                    RiiiverSDKManager.forcedDownloadApplet{ _: Boolean, _: ERError? ->
                        // 既に認証済みの場合は次の画面へ - If the user have already authenticated, goto next ViewController.
                        transitionFragment(IiideaListFragment())
                    }
                }
            }
        }
    }

    fun startWebLogin() {
        // Web認証画面表示  -  Display the webview for the authentication.
        RiiiverSDKManager.startWebLogin(object : IERAuthWebLoginCallback {
            override fun openUrl(uri: Uri?) {
                Timber.d("openUrl in")
                val intent = Intent(Intent.ACTION_VIEW, uri)
                startActivity(intent)
            }

            override fun onError(error: Exception?) {
                if(error != null) {
                    Timber.e("onError in: error %s", error.message)
                }
            }
        })
    }

    private fun transitionFragment(fragment: Fragment) {
        val transaction = supportFragmentManager.beginTransaction()
        transaction.add(R.id.container, fragment)
        transaction.commit()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        Timber.d("in")
        super.onActivityResult(requestCode, resultCode, data)
    }
}
