package jp.co.riiiver.sdk.sample.piececore

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import com.riiiver.sdk.ersdk.ERApplet
import com.riiiver.sdk.ersdk.ERTriggerBlockDelegate
import com.riiiver.sdk.ersdk.executor.ERTriggerBlockExecutor
import org.json.JSONObject
import timber.log.Timber

class TriggerPieceCoreSample2(val context: Context) : ERTriggerBlockExecutor, BroadcastReceiver() {
    var mDelegate: ERTriggerBlockDelegate? = null
    var mIdentifier: String? = null

    companion object {
        const val ACTION =
            "jp.co.riiiver.sdk.sample.piececore.intent.action.TriggerPieceCoreSample2"
    }

    override fun onReceive(context: Context?, intent: Intent?) {
        intent?.also {
            val identifier = intent.getStringExtra("identifier")
            val output = java.util.HashMap<String, Any>()

            if(intent.getStringExtra("output") != null) {
                output["triggerSampleInfo"] = intent.getStringExtra("output")!!
            }

            if (!mIdentifier.equals(identifier)) {
                // does not fire because the ID is wrong. - IDが異なるため発火しない
                return
            }

            mDelegate?.also { delegate ->
                Timber.d("TriggerPieceCoreSample2 Trigger is fired! output:%s", output)
                delegate.didTrigger(context, output)
            } ?: run {
                Timber.e("TriggerPieceCoreSample2 createTrigger:delegate is null")
            }
        }
    }

    override fun createTrigger(
        blockJsonString: String?,
        preferenceJson: JSONObject?,
        applet: ERApplet?
    ) {
        Timber.d("TriggerPieceCoreSample2 createTrigger:%s", "")

        applet?.also {
            this.mIdentifier = applet.id
        } ?: run {
            Timber.e("TriggerPieceCoreSample2 createTrigger:applet is null")
        }
    }

    override fun enable(delegate: ERTriggerBlockDelegate?): Boolean {
        Timber.d("TriggerPieceCoreSample2 enable:%s", "")

        delegate?.also {
            this.mDelegate = delegate
        } ?: run {
            Timber.e("TriggerPieceCoreSample2 createTrigger:delegate is null")
        }

        // ブロードキャストIntentの監視を開始する
        context.applicationContext.registerReceiver(this, IntentFilter(ACTION))

        return true
    }

    override fun disable() {
        Timber.d("TriggerPieceCoreSample2 disable:%s", "")
        if (mDelegate != null) {
            mDelegate = null
            // ブロードキャストIntentの監視を終了する
            context.applicationContext.unregisterReceiver(this)
        }
    }
}